//============================================================================
// Name        : GA_03_Function_identifier.cpp
// Author      : Angel Arturo Ramirez Suarez
// Version     :
// Copyright   : Baka nano
// Description : Second grade function identifier using GA.
//============================================================================

#include <iostream>
#include <sstream>
#include <fstream>
#include <string.h>
//For indentation.
#include <iomanip>
//For string manipulation.
#include <string>
#include <stdio.h>
//Used as seed.
#include <time.h>
//For power function.
#include <math.h>
#include <cmath>
//To use vectors.
#include<vector>
#include <algorithm>
//To use null.
#include <stddef.h>
//To use sleep.
#include <unistd.h>

using namespace std;
stringstream ss;

//Best candidate position.
int bestPos = 0;
//Adapation.
float sumAdapt = 0.0;

struct Ttree {
	char dato;
	Ttree *Hi;
	Ttree *Hd;
	Ttree *padre;
	int num_nodos;
	int profundidad;
	bool esRaiz;
	bool esHoja;
};

struct TIndividual{
	Ttree *tree;	//Expresion o formula.
	float adaptacion;
	float puntuacion;
	float punt_acu;
	bool elite;
};

//Available functions.
Ttree *createTree(Ttree *tree, int treeDepth, int maxDepth, Ttree *father);
void printTree(Ttree *tree, int indent);
void printPop(TIndividual *population[], int popSize, int treeDepth);
void createPop(TIndividual *population[], int popSize, int treeDepth);
void evalPop(TIndividual *population[], int popSize, int treeDepth, int maxDepth);
void selectPop(TIndividual *population[], int popSize, int treeDepth, int maxDepth);
void crossPop(struct TIndividual *population, struct TIndividual *population2, struct TIndividual *hijo1, struct TIndividual *hijo2, int lCrom, int punto_cross);
void mutPop(struct TIndividual *population[], int popSize, int lCrom, float mutRate);
void repPop(struct TIndividual *population[], int pobSize, int lCrom, float crossRate);
float calcAdapt(struct TIndividual *population, int lCrom);
string parseTree(Ttree *tree, int treeDepth, int maxDepth);
float parseString(string expression);
float function(float val);
vector<string> readVars(string fileName);
vector<string> stream_reader(istream& stream);
Ttree* setLeaf(struct Ttree *tree, int currDepth, int treeDepth, int node, int rl);
Ttree* setBranch(struct Ttree *tree, int currDepth, int treeDepth, int node, int rl);

string replaceVar(string exp);
float number(string exp);
float muldiv(string exp);
float sumres(string expressionToParse);
vector<int> sortAsc(vector<int> vector);
vector<int> findChar(string str, char toFind);
string parseSigns(string exp);
//Available expressions.
const char eqVar[4] = {'a', 'b', 'c', '2'};
const char opVar[5] = {'+', '-', '*', '/', 's'};

//To store the values for a, b & c.
string a, b, c;
vector<string> vars;

int main() {
	a = "4";
	b = "4";
	c = "7";
	string fileName = "testVals.csv";
	srand(time(NULL));
	int popSize, genSize, treeDepth;
	float mutRate, crossRate;
	popSize = genSize = treeDepth = 0;
	mutRate = crossRate = 0.0;
	cout<<"************************************************"<<endl;
	cout << "Function approximator application" << endl;
	cout<<"************************************************"<<endl;
	cout<<"Give me population size"<<endl;
	cin>>popSize;
	cout<<"Give me generations"<<endl;
	cin>>genSize;
	cout<<"Give me mutation rate"<<endl;
	cin>>mutRate;
	cout<<"Give me crossover rate"<<endl;
	cin>>crossRate;
	cout<<"Give me tree depth"<<endl;
	cin>>treeDepth;

	//Generating population.
	cout<<"************************************************"<<endl;
	cout<<"Starting algorithm"<<endl<<"Characteristics"<<endl;
	cout<<"************************************************"<<endl;
	cout<<"Generations: "<<genSize<<endl;
	cout<<"Population size: "<<popSize<<endl;
	cout<<"Mutation rate: "<<mutRate<<endl;
	cout<<"Crossover rate: "<<crossRate<<endl;

	//Declare initial population.
	TIndividual *population[popSize];
	//Allocate memory for the population.
	for(int i=0;i<popSize;i++){
		population[i] = (TIndividual *)malloc(sizeof(TIndividual));
	}
	createPop(population, popSize, treeDepth);

	//Read variable values from csv file.
	vars = readVars(fileName);
	cout<<"************************************************"<<endl;
	cout<<"Test values: "<<endl;
	for(uint i=0; i<vars.size();i+=3)
		cout<<"Var["<<i<<"] --- a("<<vars[i]<<"),b("<<vars[i+1]<<"),c("<<vars[i+2]<<")"<<endl;
	cout<<"************************************************"<<endl;

	//cout<<endl<<endl;
	//printPop(population, popSize, treeDepth);
	evalPop(population, popSize, 0, treeDepth);
	for(int i=0; i<genSize; i++){
		//Selection.
		selectPop(population, popSize, 0, treeDepth);
		//Crossover.
		repPop(population, popSize, treeDepth, crossRate);
		//Mutation.
		mutPop(population, popSize, treeDepth, mutRate);
		//Evaluation.
		evalPop(population, popSize, 0, treeDepth);
	}
	cout<<"FINISHED"<<endl;
	return 0;
}

//Reads from a specified .csv file. In this case the values for a, b & c.
vector<string> readVars(string fileName){
	ifstream file(fileName.c_str());
	//vector<float> vars;
	vector<string> vals;
	if(file.is_open()){
		while(file.good()){
			vector<string> tmp = stream_reader(file);
			vals.insert(vals.end(), tmp.begin(), tmp.end());
		}
		file.close();
	}else{
		cout<<"Unable to open file: "<<fileName<<endl;
	}

	//Turn all values into floating variables.
	/*for(uint i=0; i<vals.size(); i++){
		vars.push_back(atof(vals[i].c_str()));
	}*/
	return vals;
}

//Reads each line in the CSV file and splits from commas.
vector<string> stream_reader(istream& stream){
	vector<string> result;
	string line, cell;
	getline(stream, line);
	stringstream lineStream(line);
	while(getline(lineStream, cell, ',')){
		result.push_back(cell);
	}
	return result;
}

//Creates the expression tree.
Ttree *createTree(Ttree *tree, int treeDepth, int maxDepth, Ttree *father){
	//0) Reserve memory.
	tree = (Ttree *)malloc(sizeof(Ttree));
	//1) Decide if this will be leaf or internal node.
	int depth = treeDepth;
	int isLeaf = rand()%2;
	tree->padre = father;
	//cout<<"isLeaf: "<<isLeaf<<endl;
	//cout<<"treeDepth: "<<treeDepth<<endl;

	//if(isLeaf == 0 && depth < maxDepth){
	if(depth < maxDepth){
		//Its an internal node.
		//a) Assign a random value from the available expressions.
		int symVal = rand()%(sizeof(opVar)/sizeof(opVar[0]));
		tree->dato = opVar[symVal];		//Assign data from available operators.
		tree->profundidad = depth;		//Assign node depth.
		tree->esHoja = false;			//Its not a leaf.
		//b) Create new tree.
		if(tree->Hd == NULL)
			tree->Hd = (Ttree *)malloc(sizeof(Ttree));
			tree->Hd = createTree(tree->Hd, tree->profundidad+1, maxDepth, tree);
		if(tree->Hi == NULL)
			tree->Hi = (Ttree *)malloc(sizeof(Ttree));
			tree->Hi = createTree(tree->Hi, tree->profundidad+1, maxDepth, tree);
		return tree;
	}else{
		//Its a leaf.
		int symVal = rand()%(sizeof(eqVar)/sizeof(eqVar[0]));
		tree->dato = eqVar[symVal];
		tree->profundidad = depth;
		tree->esHoja = true;
		return tree;
	}
	return NULL;
}

void printTree(Ttree *tree, int indent=0){
	cout<<tree->dato<<","<<tree->profundidad;
	if(tree != NULL){
		if(tree->Hi != NULL)
			printTree(tree->Hi, indent+1);
		if(tree->Hd != NULL)
			printTree(tree->Hd, indent+2);
		if(indent)
			cout<<setw(indent)<<' ';
	}
}

void printPop(TIndividual *population[], int popSize, int treeDepth){
	for(int i=0; i<popSize; i++){
		cout<<endl<<"Population: "<<i<<endl;
		printTree(population[i]->tree);
	}
	cout<<endl<<"Finished"<<endl;
}

void createPop(TIndividual *population[], int popSize, int treeDepth){
	//For each of the individuals generate a tree.
	for(int i=0;i<popSize;i++){
		population[i]->tree = createTree(population[i]->tree, 0, treeDepth, NULL);
	}
}

string parseTree(Ttree *tree, int treeDepth, int maxDepth){
	string expression = (char*)&tree->dato;
	if(tree->Hi != NULL){
		string left = parseTree(tree->Hi, treeDepth, maxDepth);
		expression = left + expression;
	}
	if(tree->Hd != NULL){
		string right = parseTree(tree->Hd, treeDepth, maxDepth);
		expression = expression + right;
	}
	return expression;
}

//Replace the numbers of a, b & c with their float values.
string replaceVar(string exp){
	int temp_index=0;
	while(true){
		temp_index = exp.find("a", temp_index);
		//If it doesn't exist.
		if(temp_index==std::string::npos)
			break;
		//Check if the value is positive. If so, replace.
		if(a.find('-') != string::npos){
			//cout<<"Negative A: "<<a<<endl;
			string tmp = exp.substr(temp_index+1, exp.size());
			exp.replace(temp_index, (signed)strlen(a.c_str()), a);
			exp.replace(temp_index+strlen(a.c_str()), strlen(tmp.c_str()), tmp);
		}else{
			//cout<<"A: "<<a<<endl;
			exp.replace(temp_index, (signed)strlen(a.c_str()), a);
		}
		temp_index+=(signed)strlen(a.c_str());
	}
	temp_index=0;
	while(true){
		temp_index = exp.find("b", temp_index);
		//If it doesn't exist.
		if(temp_index==std::string::npos)
			break;
		//Check if the value is positive. If so, replace.
		if(b.find('-') != string::npos){
			//cout<<"Negative B: "<<b<<endl;
			string tmp = exp.substr(temp_index+1, exp.size());
			exp.replace(temp_index, (signed)strlen(b.c_str()), b);
			exp.replace(temp_index+strlen(b.c_str()), strlen(tmp.c_str()), tmp);
			//exp.replace(temp_index+2, (signed)strlen(temp.c_str()), temp);
		}else{
			//cout<<"B: "<<b<<endl;
			exp.replace(temp_index, (signed)strlen(b.c_str()), b);
		}
		temp_index+=(signed)strlen(b.c_str());
	}
	temp_index=0;
	while(true){
		temp_index = exp.find("c", temp_index);
		//If it doesn't exist.
		if(temp_index==std::string::npos)
			break;
		//Check if the value is positive. If so, replace.
		if(c.find('-') != string::npos){
			//cout<<"Negative C: "<<c<<endl;
			string tmp = exp.substr(temp_index+1, exp.size());
			exp.replace(temp_index, (signed)strlen(c.c_str()), c);
			exp.replace(temp_index+strlen(c.c_str()), strlen(tmp.c_str()), tmp);
		}else{
			//cout<<"C: "<<c<<endl;
			exp.replace(temp_index, (signed)strlen(c.c_str()), c);
		}
		temp_index+=(signed)strlen(c.c_str());
	}
	return exp;
}

//This function deletes all group of signs.
string parseSigns(string exp){
	string str = exp;
	//Locate all + signs.
	vector<int> charLocationS = findChar(str, '+');
	vector<int> charLocationM = findChar(str, '-');
	vector<int> charPos;
	if(charLocationS.size() > 0){
		if(charPos.empty())
			charPos = charLocationS;
		charPos.insert(charPos.end(), charLocationS.begin(), charLocationS.end());
	}
	if(charLocationM.size() > 0){
		if(charPos.empty())
			charPos = charLocationM;
		charPos.insert(charPos.end(), charLocationM.begin(), charLocationM.end());
	}
	vector<int> signs = sortAsc(charPos);
	uint strLen = signs.size()+1;
	for(uint i=0; i<strLen-1; i++){
		if(exp[signs[i]-1] == '+' && exp[signs[i]] == '+'){
			exp.replace(signs[i], exp.size(), exp.substr(signs[i]+1));
			i=0;
		}
		else if((exp[signs[i]-1] == '+' && exp[signs[i]] == '-') || (exp[signs[i]-1] == '-' && exp[signs[i]] == '+')){
			exp.replace(signs[i], exp.size(), exp.substr(signs[i]+1));
			//Replace + with - sign (+ - = -).
			exp.replace(signs[i]-1, 1, "-");
			i=0;
		}
		else if(exp[signs[i]-1] == '-' && exp[signs[i]] == '-'){
			exp.replace(signs[i], exp.size(), exp.substr(signs[i]+1));
			//Replace + with - sign (+ - = -).
			exp.replace(signs[i]-1, 1, "+");
			i=0;
		}
	}
	return exp;
}

//Convert into number.
float number(string exp){
	if(exp.empty()){
		return 0.0;
	}else{
		float num = atof(exp.c_str());
		return num;
	}
}

float muldiv(string exp){
	//cout<<"Muldiv exp: "<<exp<<endl;
	float result = 0.0;
	//Find all *, sqrt and /.
	//Find all square roots.
	vector <int> charLocationS = findChar(exp, 's');
	vector <int> charLocationM = findChar(exp, '*');
	vector <int> charLocationD = findChar(exp, '/');
	vector <int> vecPos = charLocationS;
	if(charLocationS.size() > 0){
		if(vecPos.empty())
			vecPos = charLocationS;
		vecPos.insert(vecPos.end(), charLocationS.begin(), charLocationS.end());
	}
	if(charLocationM.size() > 0){
		if(vecPos.empty())
			vecPos = charLocationM;
		vecPos.insert(vecPos.end(), charLocationM.begin(), charLocationM.end());
	}
	//Rearrange in ascending order.
	vector <int> sentence = sortAsc(vecPos);
	int idx = 0;
	if(sentence.size()<1){
		return(number(exp));
	}
	result = number(exp.substr(idx, sentence[0]));
	for(uint i=0; i<sentence.size(); i++){
		idx = sentence[i];
		if(exp[sentence[i]] == 's'){
			//Apply square root.
			if(number(exp.substr(sentence[i]+1, sentence[i+1]-sentence[i]-1)) >= 0){
				result += sqrt(number(exp.substr(sentence[i]+1, sentence[i+1]-sentence[i]-1)));
			}
			else{
				//Gets a big number, hence a low quality solution.
				result += 100;
			}
			idx = sentence[i]+1;
		}else if(exp[sentence[i]]=='/'){
			//Apply division.
			if(number(exp.substr(sentence[i]+1, sentence[i+1]-sentence[i]-1)) >= 0){
				result /= number(exp.substr(sentence[i]+1, sentence[i+1]-sentence[i]-1));
			}else{
				//Gets a big number, hence a low quality solution.
				result += 100;
			}
		}else if(exp[sentence[i]]=='*'){
			//Apply multiplication.
			result *= number(exp.substr(sentence[i]+1, sentence[i+1]-sentence[i]-1));
		}
		else{
			result = number(exp);
		}
	}
	return result;
}

//Implements sum and res operations recursively.
float sumres(string expressionToParse){
	//Eliminate repeated signs.
	expressionToParse = parseSigns(expressionToParse);
	cout<<"Parsed: "<<expressionToParse<<endl;
	float result = 0.0;//To store result of arithmetic operation.
	//First we must determine all + and -'s in the string.
	vector <int> charLocationS = findChar(expressionToParse, '+');
	vector <int> charLocationM = findChar(expressionToParse, '-');
	vector <int> vecPos = charLocationS;

	if(charLocationS.size() > 0){
		if(vecPos.empty())
			vecPos = charLocationS;
		vecPos.insert(vecPos.end(), charLocationS.begin(), charLocationS.end());
	}
	if(charLocationM.size() > 0){
		if(vecPos.empty())
			vecPos = charLocationM;
		vecPos.insert(vecPos.end(), charLocationM.begin(), charLocationM.end());
	}
	//Rearrange in ascending order.
	vector <int> sentence = sortAsc(vecPos);
	int idx = 0;
	//If it didn't find any sum or res.
	if(sentence.size()<1)
		result = muldiv(expressionToParse);

	result = muldiv(expressionToParse.substr(idx, sentence[0]));
	//cout<<"Result "<<result<<" Index:"<<idx<<", "<<sentence[0]-idx<<endl;
	//Else sum or res the values.
	for(uint i=0; i<sentence.size(); i++){
		idx = sentence[i];
		if(expressionToParse[sentence[i]] == '+'){
			//cout<<expressionToParse[sentence[i]]<<endl;
			//cout<<expressionToParse.substr(sentence[i]+1, sentence[i+1]-sentence[i]-1)<<endl;
			//cout<<"IDX1: "<<sentence[i]+1<<", IDX2: "<<sentence[i+1]-sentence[i]-1<<endl;
			result += muldiv(expressionToParse.substr(sentence[i]+1, sentence[i+1]-sentence[i]-1));
		}else{
			//cout<<expressionToParse[sentence[i]]<<endl;
			//cout<<expressionToParse.substr(sentence[i]+1, sentence[i+1]-sentence[i]-1)<<endl;
			//cout<<"IDX1: "<<sentence[i]+1<<", IDX2: "<<sentence[i+1]-sentence[i]-1<<endl;
			result -= muldiv(expressionToParse.substr(sentence[i]+1, sentence[i+1]-sentence[i]-1));
		}
	}
	return result;
}

vector<int> sortAsc(vector<int> vector){
	if(vector.size()>0){
		uint swap;
		for (uint c = 0 ; c < ( vector.size() - 1 ); c++){
			for (uint d = 0 ; d < vector.size() - c - 1; d++){
			  if (vector[d] > vector[d+1]){
				  swap       = vector[d];
				  vector[d]   = vector[d+1];
				  vector[d+1] = swap;
			  }
			}
		}
	}
	return vector;
}

vector<int> findChar(string str, char toFind){
	vector<int> charLocation;
	for(uint i=0;i<str.size(); i++){
		if(str[i] == toFind)
			charLocation.push_back(i);
	}
	return charLocation;
}

float parseString(string exp){
	exp = replaceVar(exp);
	float result = sumres(exp);
	return result;
}

//Evaluates the functions.
float function(float val){
	//Evaluate the variable for the 2nd degree equation.
	float res = atof(a.c_str())*pow(val, 2) + atof(b.c_str())*val + atof(c.c_str());
	return res;
}

void selectPop(TIndividual *population[], int popSize, int treeDepth, int maxDepth){
	float prob;
	int pos_super;
	int sel_super_[popSize];
	TIndividual *pob_aux[popSize];
	for(int i=0;i<popSize;i++){
		//Generate random numbers between 0 and 1.
		prob = ((double) rand() / (RAND_MAX));
		pos_super = 0;
		while((prob > population[pos_super]->punt_acu)&&(pos_super < popSize)){
			pos_super++;
		}
		sel_super_[i] = pos_super;
	}
	//Generation of intermediate population.
	for(int i=0;i<popSize;i++){
		pob_aux[i] = population[sel_super_[i]];
	}
	for(int i=0;i<popSize;i++){
		population[i] = pob_aux[i];
	}
}

void evalPop(TIndividual *population[], int popSize, int treeDepth, int maxDepth){
	cout<<endl<<"****************************************************"<<endl;
	cout<<"Evaluating population."<<endl;
	cout<<"****************************************************"<<endl;
	//Reset accumulated adaptation value.
	sumAdapt = 0.0;
	//Best chromosome position.
	int pos=0;
	float punt_acum = 0.0;
	float best_adapt = 0.0;

	//1) Substitude population value for expression.
	string expression[popSize];
	float eval[popSize];//Stores the evaluation value.
	float cost1;//[popSize];//, cost2[popSize];

	for(uint i=0;i<popSize;i++){
		expression[i] = parseTree(population[i]->tree, 0, treeDepth);
		cout<<"Expression: "<<i<<"\t---\t"<<expression[i]<<endl;
		//2) Evaluate.
		//Evaluate for each of the fields in the CSV.
		for(uint k=0; k<vars.size(); k+=3){
			a = vars[k];
			b = vars[k+1];
			c = vars[k+2];
			//a) Convert from string to mathematic operator.
			eval[i] = parseString(expression[i]);
			cout<<"Result (floating point): "<<eval[i]<<endl;
			//3) Check if expression is close to zero. Divided by 100 to avoid huge numbers.
			cost1 += function(eval[i])/100;
			//cout<<"Cost: "<<cost1<<endl;
		}
		//Reset cost value to avoid dirty data.
		population[i]->adaptacion = cost1;
		cout<<"************************************************"<<endl;
		cost1 = 0;
	}
	//We use the first value as error to overcome.
	sumAdapt = population[0]->adaptacion;

	for(int i=0;i<popSize;i++){
		cout<<"Function adaptation: "<<population[i]->adaptacion;
		sumAdapt = sumAdapt + population[i]->adaptacion;
		cout<<"\t\tAcumulated: "<<sumAdapt<<endl;
		//Compare the adaptation of actual value to select the best candidate.
		if(abs(population[i]->adaptacion) < best_adapt){
			pos = i;
			best_adapt = population[i]->adaptacion;
		}
	}
	cout<<endl<<"Position of the best individual: "<<pos<<endl;
	for(int i=0;i<popSize;i++){
		population[i]->puntuacion = population[i]->adaptacion / sumAdapt;
		population[i]->punt_acu = population[i]->puntuacion + punt_acum;
		punt_acum = punt_acum + population[i]->puntuacion;
	}
	bestPos = pos;
}

//Calculates a population adaptation value.
float calcAdapt(struct TIndividual *population, int treeDepth){
	float cost;
	string expression = parseTree(population->tree, 0, treeDepth);
	for(uint k=0; k<vars.size(); k+=3){
		a = vars[k];
		b = vars[k+1];
		c = vars[k+2];
		//a) Convert from string to mathematic operator.
		float eval = parseString(expression);
		cout<<"Result (floating point): "<<eval<<endl;
		//3) Check if expression is close to zero. Divided by 100 to avoid huge numbers.
		cost += function(eval)/100;
		//cout<<"Cost1: "<<cost<<endl;//[idx]<<endl;
	}
	return cost;
}

void repPop(struct TIndividual *population[], int pobSize, int treeDepth, float crossRate){
	cout<<endl<<"****************************************************"<<endl;
	cout<<"Initiating crossover."<<endl;
	cout<<"****************************************************"<<endl;
	int selCross[pobSize];
	int num_sel_cross = 0;
	float prob;
	int punto_cross;
	TIndividual *hijo1 = (TIndividual *)malloc(sizeof(TIndividual));
	//Allocate space for every element inside structure.
	hijo1->tree = createTree(hijo1->tree, 0, treeDepth, NULL);

	TIndividual *hijo2 = (TIndividual *)malloc(sizeof(TIndividual));
	//Allocate space for every element inside structure.
	hijo2->tree = createTree(hijo2->tree, 0, treeDepth, NULL);

	//Selecting individuals to cross.
	for(int i=0;i<pobSize;i++){
		//Generate random numbers between 0 and 1.
		prob = ((double) rand() / (RAND_MAX));
		if(prob<crossRate){
			selCross[num_sel_cross] = i;
			num_sel_cross++;
		}
	}
	if((num_sel_cross%2) == 1){
		num_sel_cross--;
	}
	//Cross individuals at random point.
	punto_cross = ((double) rand() / (RAND_MAX));
	for(int i=0;i<num_sel_cross-1;i=i+2){
		crossPop(population[selCross[i]], population[selCross[i+1]], hijo1, hijo2, treeDepth, punto_cross);
		//New individuals replace the fathers.
		population[selCross[i]] = hijo1;
		population[selCross[i+1]] = hijo2;
		//cout<<"Crossed: "<<i<<"\t---\t"<<population[i]<<endl;
	}
}

void crossPop(struct TIndividual *population, struct TIndividual *population2, struct TIndividual *hijo1, struct TIndividual *hijo2, int treeDepth, int punto_cross){
	for(int i=0;i<punto_cross;i++){
		hijo1->tree[i] = population->tree[i];
		hijo2->tree[i] = population2->tree[i];
	}
	for(int i=punto_cross;i<treeDepth;i++){
		hijo1->tree[i] = population2->tree[i];
		hijo2->tree[i] = population->tree[i];
	}

	hijo1->adaptacion = calcAdapt(hijo1, treeDepth);
	hijo2->adaptacion = calcAdapt(hijo2, treeDepth);
}

//currDepth = actual depth of the function, treeDepth = max depth allowed,
//node = desired, rl = right(1) or left(0).
Ttree* setLeaf(struct Ttree *tree, int currDepth, int treeDepth, int node, int rl){
	if(currDepth < node && currDepth < treeDepth){
		if(rl <= 0)
			setLeaf(tree->Hi, currDepth+1, treeDepth, node, rl);
		else
			setLeaf(tree->Hd, currDepth+1, treeDepth, node, rl);
	}else{
		//Randomly select one of the operators and replace.
		if(rl <= 0){
			int newOp = (sizeof(opVar)/sizeof(opVar[0]));
			tree->dato = opVar[newOp];
		}
		else{
			int newOp = (sizeof(opVar)/sizeof(opVar[0]));
			tree->dato = opVar[newOp];
		}
	}
	return tree;
}

//currDepth = actual depth of the function, treeDepth = max depth allowed,
//node = desired, rl = right(1) or left(0).
Ttree *setBranch(struct Ttree *tree, int currDepth, int treeDepth, int node, int rl){
	if(currDepth < node && currDepth < treeDepth){
		if(rl <= 0)
			setBranch(tree->Hi, currDepth+1, treeDepth, node, rl);
		else
			setBranch(tree->Hd, currDepth+1, treeDepth, node, rl);
	}else{
		//Randomly select one of the operators and replace.
		if(rl <= 0){
			int newOp = (sizeof(eqVar)/sizeof(eqVar[0]));
			tree->dato = eqVar[newOp];

		}
		else{
			int newOp = (sizeof(eqVar)/sizeof(eqVar[0]));
			tree->dato = eqVar[newOp];
		}
	}
	return tree;
}

void mutPop(struct TIndividual *population[], int popSize, int treeDepth, float mutRate){
	cout<<endl<<"****************************************************"<<endl;
	cout<<"Initiating mutation."<<endl;
	cout<<"****************************************************"<<endl;
	bool mutated = false;
	for(int i=0;i<popSize;i++){
		mutated = false;
		//Determine if it will be mutated or not.
		float prob = ((double) rand() / (RAND_MAX));
		//******************************************************************
		//******************************************************************
		//prob = 0.0;//Only for debugging
		//******************************************************************
		//******************************************************************
		if(prob < mutRate){
			int rl;
			//Determine if it will implement exchange, operator or variable change.
			//int operation = rand()%5+1;
			int operation = rand()%2;//Currently only operator and variable change
			//are available.
			//******************************************************************
			//******************************************************************
			//int operation = 2;//Only for debugging.
			//******************************************************************
			//******************************************************************
			//cout<<"Operation to use: "<<operation<<endl;
			switch(operation){
				case 0:
					//cout<<"Operator change"<<endl;
					//Change one of the variables randomly.
					//Select right or left.

					rl = rand()%2;
					//cout<<"RL: "<<rl<<endl;
					if(rl == 0){
						//Randomly select a node between 0 to treeDepth.
						int node = rand()%treeDepth;
						setBranch(population[i]->tree, 0, treeDepth, node, 0);
						//cout<<"Node depth: "<<node<<endl;
					}
					else{
						//Randomly select a node between 0 to treeDepth.
						int node = rand()%treeDepth;
						setBranch(population[i]->tree, 0, treeDepth, node, 1);
						//cout<<"Node depth: "<<node<<endl;
					}
					break;
				case 1:
					//cout<<"Variable change"<<endl;
					//Change one of the operators randomly.
					//Select right or left.
					rl = rand()%2;
					//cout<<"RL: "<<rl<<endl;
					if(rl == 0){
						int node = treeDepth;//Node == treeDepth AKA leaf.
						setLeaf(population[i]->tree, 0, treeDepth, node, 0);
					}
					else{
						int node = treeDepth;//Node == treeDepth AKA leaf.
						setLeaf(population[i]->tree, 0, treeDepth, node, 1);
					}
					//cout<<"Tree depth: "<<treeDepth<<endl;
					break;
				case 2:
					cout<<"Change tree branch"<<endl;
					//Change a whole branch.
					cout<<"Tree depth: "<<treeDepth<<endl;
					break;
				default:
					cout<<"Exchange"<<endl;
					//To check.
					cout<<"Tree depth: "<<treeDepth<<endl;
					break;
			}
			mutated = true;
		}
		if(mutated){
			//cout<<"Mutated"<<endl;
			population[i]->adaptacion = calcAdapt(population[i], treeDepth);
		}
	}
	//Print tree, only for debugging purposes.
	//printPop(population, popSize, treeDepth);
}
